require 'naranya_ecm-sdk/version'

require 'active_support/time'
require 'active_support/dependencies/autoload'
require 'active_support/core_ext/hash/indifferent_access.rb'
require 'yaml'

require 'ncontent-sdk'

module NaranyaEcm

  extend ActiveSupport::Autoload

  autoload_under 'rest' do
    autoload :Client
  end

  # == NaranyaEcm \Rest Module
  #
  # <tt>NaranyaEcm::Rest</tt> provides the REST persistance and querying layer
  # that connects the NaranyaEcm models with the Naranya CaaS REST services.
  module Rest
    extend ActiveSupport::Autoload
    autoload :Errors
    autoload :Persistence
    autoload :Associations
    autoload :Model
    autoload :FinderMethods
    autoload :Relation
    autoload :AssociationRelation
  end

  autoload_under 'models' do
    autoload :Category
    autoload :Content
    autoload :ContentVersion
    autoload :DownloadAuthorization
    autoload :MediaProfile
    autoload :MediaProcess
    autoload :MediaResource
    autoload :Notification
  end

  module Behaviors
    extend ActiveSupport::Autoload
    autoload :Localizable
    autoload :Timestampable
    autoload :MediaMethods
  end

  autoload_under 'lifecycles' do
    autoload :ContentLifecycle
  end

  # == NaranyaEcm \Search Module
  #
  # <tt>NaranyaEcm::Search</tt> implements classes and methods for querying the Naranya CaaS Search engine.
  module Search
    extend ActiveSupport::Autoload
    autoload :Hit
    autoload :Methods
    autoload :Query
    autoload :Results
  end

  def self.document_module; NaranyaEcm::Rest::Model; end
  def self.timestamp_module; NaranyaEcm::Behaviors::Timestampable; end

end
