require 'active_support/notifications'

module NaranyaEcm
  module Rest
    module Notifications
      extend ActiveSupport::Concern

      module ClassMethods
        def as_notification_payload
          { class: self.name }
        end
      end

      def as_notification_payload
        instance_payload = self.class.as_notification_payload.merge({ id: self.id })

        # Include belongs_to references:
        self.class.reflect_on_all_associations.values.select do |association|
          association.class.name.demodulize == 'BelongsTo'
        end.map(&:foreign_key).each do |reference_name|
          instance_payload[reference_name] = self.public_send reference_name
        end

        instance_payload
      end

      def start_notify_state_change!
        ActiveSupport::Notifications.instrumenter
        .start("ncontent_sdk.#{aasm.current_event.to_s.tr('!', '').underscore}", self.as_notification_payload)
      end

      def finish_notify_state_change!
        ActiveSupport::Notifications.instrumenter
        .finish("ncontent_sdk.#{aasm.current_event.to_s.tr('!', '').underscore}", self.as_notification_payload)
      end
    end
  end
end
