require 'naranya_ecm/rest/errors'

module NaranyaEcm::Rest
  class Relation
    include Enumerable
    delegate :to_yaml, :all?, :each, :first, :last, :to_ary, to: :to_a

    attr_reader :klass, :conditions, :elements

    def initialize(given_klass, given_conditions = {})
      @klass, @conditions = given_klass, given_conditions.with_indifferent_access
    end

    def [](n)
      to_a[n]
    end

    def where(given_conditions = {})
      spawn_chain(given_conditions)
    end

    def limit(given_limit)
      spawn_chain(limit: given_limit.to_i)
    end

    def to_a
      materialize! if @elements.nil?
      elements
    end

    def fetch_from_server
      NContent::SDK::RESTClient.get do |req|
        req.url "#{klass.path}.json", conditions
      end.body
    end

    def load(given_list)
      # Revisar que todos los elementos sean Hash o la clase asociada:
      if given_list.count > 0 && given_list.map{ |e| e.is_a?(Hash) || e.is_a?(@klass) }.reduce { |res, e| res && e } == false
        raise "Type Mismatch: some elements are not a hash nor associated_class"
      end

      @elements = given_list.map do |element|
        if element.is_a?(Hash)
          loaded_element = @klass.send :_load_from_fetched_attributes, element
          loaded_element
        else
          element
        end
      end
    end

    private
      def spawn_chain(given_conditions = {})
        self.class.new klass, conditions.merge(given_conditions)
      end

      def materialize!
        @elements = fetch_from_server.map do |s|
          resource = klass.send :_load_from_fetched_attributes, s
          resource.instance_variable_set("@new_resource", false)
          resource
        end
      end

  end
end
