# Require the Railtie if loaded from Rails app:
require 'ncontent/sdk/railtie' if defined? Rails
require 'active_support/core_ext/module'

module NContent

  module SDK

    extend ActiveSupport::Autoload

    autoload :Config,     "ncontent/sdk/config"
    autoload :RESTClient, "ncontent/sdk/rest_client"
    autoload :Storage,    "ncontent/sdk/storage"

    def self.config
      @@config ||= NContent::SDK::Config.new
    end

    def self.configure(config_hash = {})
      if block_given?
        yield config
      else
        config_hash.each do |config_key, config_value|
          if config.respond_to? "#{config_key}="
            config.public_send "#{config_key}=", config_value
          end
        end
      end
    end

    mattr_writer :logger
    def self.logger
      @@logger ||= Logger.new($stdout)
    end

    def self.storage
      @@storage ||= ::NContent::SDK::Storage.new
    end

    def self.root_directory
      @@root_directory ||= storage.directories.get(config.bucket_name)
    end

    mattr_writer :cache
    def self.cache
      @@cache ||= ActiveSupport::Cache.lookup_store *config.cache
    end

  end

end
