require "active_support/configurable"
require 'i18n/core_ext/hash'
require 'yaml'
require 'erb'

module NContent
  module SDK

    class Config

      include ActiveSupport::Configurable

      config_accessor :api_adapter do
        ENV.fetch 'NCONTENT_API_ADAPTER', 'net_http'
      end

      config_accessor :api_host do
        ENV['NCONTENT_API_HOST'] || ENV['NARANYA_ECM_SITE'] || 'http://proy-cms-1.herokuapp.com'
      end

      config_accessor :api_key do
        ENV['NCONTENT_API_KEY'] || ENV['NARANYA_ECM_API_KEY']
      end

      config_accessor :api_secret do
        ENV['NCONTENT_API_SECRET'] || ENV['NARANYA_ECM_API_SECRET']
      end

      # API Call Timeout in seconds:
      config_accessor :api_call_timeout do
        ENV.fetch("NCONTENT_API_CALL_TIMEOUT", '30').to_i
      end

      # API Call Open Timeout in seconds:
      config_accessor :api_call_open_timeout do
        ENV.fetch("NCONTENT_API_CALL_OPEN_TIMEOUT", '30').to_i
      end

      config_accessor :bucket_provider do
        ENV['NCONTENT_BUCKET_PROVIDER'] || ENV['NARANYA_ECM_STORAGE_PROVIDER'] || 'AWS'
      end

      # (AWS) Bucket name
      config_accessor :bucket_name do
        ENV['NCONTENT_BUCKET_NAME'] || ENV['NARANYA_ECM_STORAGE_DIR']
      end

      config_accessor :bucket_api_key do
        ENV['NCONTENT_BUCKET_API_KEY'] || ENV['NARANYA_ECM_STORAGE_KEY']
      end

      config_accessor :bucket_api_secret do
        ENV['NCONTENT_BUCKET_API_SECRET'] || ENV['NARANYA_ECM_STORAGE_SECRET']
      end

      config_accessor(:media_profiles) { {} }

      config_accessor (:extra_attributes) { {} }

      config_accessor(:cache) { :memory_store }

      config_accessor :client_notify_url do
        ENV.fetch 'NARANYA_ECM_CLIENT_NOTIFY_URL',
          ENV.fetch('NCONTENT_CLIENT_NOTIFY_URL', 'http://www.example.com/ncontent/notifications')
      end

      config_accessor(:notification_processing_modules) { [] }

      config_accessor :state_event_delay do
        (ENV.fetch 'NCONTENT_STATE_EVENT_DELAY', "0").to_f
      end

      def new_relic_agent_available?
        defined?(::NewRelic) && defined?(::NewRelic::Agent)
      end

      def self.parse_yaml(yaml_path)
        YAML.load(ERB.new(File.read(yaml_path)).result) if File.exist?(yaml_path)
      end

      delegate :parse_yaml, to: :class

      protected

      end

  end
end
