require "benchmark"

module NContent
  module SDK
    module FaradayMiddleware
      # Parses the responses to models:
      class RESTAPICallBenchmark < Faraday::Middleware

        def call(request_env)

          # Get the start times:
          t0 = Process.times
          r0 = if defined?(Benchmark::BENCHMARK_CLOCK)
            Process.clock_gettime(Benchmark::BENCHMARK_CLOCK)
          else
            Time.now
          end

          @app.call(request_env).on_complete do |response_env|

            response_env[:ncontent_rest_api_response_datetime] = Time.parse(
              response_env.response_headers["Date"]
            ) if response_env.response_headers.key? "Date"

            # Get the finish times:
            t1 = Process.times
            r1 = if defined?(Benchmark::BENCHMARK_CLOCK)
              Process.clock_gettime(Benchmark::BENCHMARK_CLOCK)
            else
              Time.now
            end

            response_env[:ncontent_rest_api_calling_tms] = ::Benchmark::Tms.new(
              t1.utime  - t0.utime,
              t1.stime  - t0.stime,
              t1.cutime - t0.cutime,
              t1.cstime - t0.cstime,
              r1 - r0,
              "api_calling"
            )


          end
        end

      end
    end
  end
end
